@@ -16,7 +16,6 @@
 
 import numpy as np
 import sageopt.coniclifts as cl
-from collections import defaultdict
 from sageopt.symbolic.signomials import Signomial
 import warnings
 
@@ -56,45 +55,29 @@ def standard_poly_monomials(n):
 
 class Polynomial(Signomial):
     """
-    A symbolic representation for polynomials which are sparse in the monomial basis.
-    The constructor for this class can be called in two different ways, and the arguments
-    to the constructor have names which reflect the different possibilities.
+    A concrete representation for a multivariate polynomial
+    :math:`x \\mapsto \\sum_{i=1}^m c_i \\prod_{j=1}^n x_j^{\\alpha_{ij}}`.
+
+    Polynomial objects provide the same operator overloading as the Signomial class.
+    In particular, Polynomials implement ``+``, ``-``, ``*``, ``/``, and ``**``.
 
     Parameters
     ----------
 
-    alpha_maybe_c : dict or ndarray
+    alpha : ndarray
 
-         If ``alpha_maybe_c`` is a dict, then it must be a dictionary from tuples-of-numbers to
-         scalars. The keys will be converted to rows of a matrix which we call ``alpha``, and
-         the values will be assembled into a vector which we call ``c``.
+        The returned Polynomial will have one monomial for each row in alpha. The number
+        ``alpha[i, j]`` represents the power of variable ``j`` on the i-th monomial.
 
-         If ``alpha_maybe_c`` is an ndarray, then the argument ``c`` must also be an ndarray,
-         and ``c.size`` must equal the number of rows in ``alpha_maybe_c``.
+    c : ndarray
 
-    c : None or ndarray
-
-        This value is only used when ``alpha_maybe_c`` is an ndarray. In that case, this
-        Polynomial represents ``lambda x: c @ np.prod(np.power(alpha_maybe_c, x))``.
+        The number ``c[i]`` os the coefficient on the i-th monomial, where the i-th
+        monomial is defined by ``alpha[i, :]``.
 
     Examples
     --------
 
-    There are two ways to call the Polynomial constructor.
-
-    The first way is to specify a dictionary from tuples to scalars. Each key-value pair of the dictionary
-    represents a monomial appearing in this polynomial. The key tuples must all be of some common length ``n``.
-    If ``(a, coeff)`` is a key-value pair in this dictionary, then the Polynomial includes an additive term
-    ``lambda x: coeff * np.prod(np.power(a, x))``. ::
-
-        alpha_and_c = {(1,): 2}
-        f = Polynomial(alpha_and_c)
-        print(f(1))  # equal to 2.
-        print(f(-3))  # equal to -6.
-
-    The second way is to specify two arguments. In this case the first argument ``alpha`` is an ndarray of
-    exponent vectors, where ``alpha[i, j]`` is the power of variable ``j`` in monomial ``i``. The second argument
-    is a numpy array ``c``, where ``c[i]`` is the coefficient on the i-th monomial defined by ``alpha``. ::
+    There are three ways to make Polynomial objects. The first way is to call the constructor::
 
         alpha = np.array([[1, 0], [0, 1], [1, 1]])
         c = np.array([1, 2, 3])
@@ -103,29 +86,30 @@ class Polynomial(Signomial):
         val = f(x)  # val = 1 * (-4) + 2 * (7) + 3 * (-4 * 7)
         print(val)  # -74
 
-    Attributes
-    ----------
+    You can also use ``Polynomial.from_dict``, which maps exponent vectors (represented as
+    tuples) to scalars::
 
-    n : int
-        The dimension of the space over which this Polynomial is defined. The number of columns in ``alpha``,
-        and the length of tuples appearing in the dictionary ``alpha_c``.
+        alpha_and_c = {(1,): 2}
+        f = Polynomial.from_dict(alpha_and_c)
+        print(f(1))  # equal to 2.
+        print(f(-3))  # equal to -6.
 
-    m : int
-        The number of terms needed to express this Polynomial in the standard monomial basis.
-        The number of rows in  ``alpha``.  The length of the dictionary ``alpha_c``.
+    The final way to construct a Polynomial is with algebraic syntax, like::
 
-    alpha : ndarray
-        Has shape ``(m, n)``. Entry ``alpha[i,j]`` is the power of an implicit variable ``x[j]``
-        appearing in the i-th monomial for this Polynomial. The i-th monomial, in turn, has coefficient ``c[i]``.
+        x = standard_poly_monomials(3)
+        f = (x[0] ** 2) * x[2] + 4 * x[1] * x[2] * x[0]
+        z = np.array([1, 2, 3])
+        val = f(z)  # val = (1**2) * 3 + 4 * (2 * 3 * 1)
+        print(val)  # 27.
 
-    c : ndarray
-        Has shape ``(m,)``. The scalar ``c[i]`` is this Polynomial's coefficient on the basis function
-        ``lambda x: np.prod(np.power(alpha[i, :], x))``. It is possible to have ``c.dtype == object``, to allow for
-        coniclifts Variables.
+    Polynomial objects are not limited to numeric problem data for ``alpha`` and ``c``.
+    In fact, it's very common to have ``c`` contain a coniclifts Expression. For example,
+    if we started with a Polynomial ``f`` and then updated ::
 
-    alpha_c : defaultdict
-        The keys of ``alpha_c`` are tuples of length ``n``, containing real numeric types (e.g int, float).
-        These tuples correspond to rows in ``alpha``.
+        gamma = sageopt.coniclifts.Variable()
+        f =  f - gamma
+
+    then ``f.c`` would be a coniclifts Expression depending on the variable ``gamma``.
 
     Notes
     -----
@@ -133,10 +117,13 @@ class Polynomial(Signomial):
     polynomials and signomials are identical. However it is important to remember that polynomials and signomials
     evaluate in very different ways.
 
+    Polynomial objects have a dictionary attribute called ``metadata``. You can store any information
+    you'd like in this dictionary. However, the information in this dictionary will not automatically be
+    propogated when creating new Polynomial objects (as happens when performing arithmetic on Polynomials).
     """
 
-    def __init__(self, alpha_maybe_c, c=None):
-        Signomial.__init__(self, alpha_maybe_c, c)
+    def __init__(self, alpha, c):
+        Signomial.__init__(self, alpha, c)
         if not np.all(self.alpha % 1 == 0):  # pragma: no cover
             raise RuntimeError('Exponents must belong the the integer lattice.')
         if not np.all(self.alpha >= 0):  # pragma: no cover
@@ -145,6 +132,74 @@ class Polynomial(Signomial):
         self._sig_rep_constrs = []
         pass
 
+    @property
+    def n(self):
+        """
+        The dimension of the space over which this Polynomial is defined;
+        this Polynomial accepts inputs in :math:`\\mathbb{R}^{n}`.
+        """
+        return self._n
+
+    @property
+    def m(self):
+        """
+        The number of monomial basis functions :math:`x \\mapsto \\prod_{j=1}^n x_j^{a_j}`
+        used by this Polynomial.
+        """
+        return self._m
+
+    @property
+    def alpha(self):
+        """
+        Has shape ``(m, n)``. Each row specifies the exponents of a monomial which
+        appear in this Polynomial. The rows are ordered for consistency with the property ``c``.
+        """
+        return self._alpha
+
+    @property
+    def c(self):
+        """
+        Has shape ``(m,)``. The scalar ``c[i]`` is this Polynomial's coefficient on the
+        basis function ``lambda x: np.prod(np.power(x, alpha[i, :]))``.
+        """
+        return self._c
+
+    @property
+    def alpha_c(self):
+        """
+        The keys of ``alpha_c`` are tuples of length ``n``, containing real numeric types (e.g int, float).
+        These tuples define monomial exponents. This Polynomial could be evaluated by the code snippet
+        ``lambda x: np.sum([ alpha_c[a] * np.prod(np.power(x,a)) for a in alpha_c])``.
+        """
+        if self._alpha_c is None:
+            self._build_alpha_c_dict()
+        return self._alpha_c
+
+    @property
+    def sig_rep(self):
+        """
+        Return the signomial representative of the current Polynomial, as well as a list of constraints needed
+        to enforce the relationship between the current Polynomial and the signomial representative.
+
+        Returns
+        -------
+        sr : Signomial
+            If this Signomial is globally nonnegative, then the current Polynomial is also globally nonnegative.
+
+        sr_cons : list of coniclifts Constraints
+            If the current Polynomial has nonconstant coefficients (i.e. some entries of ``c`` are coniclifts
+            Variables), then ``sr`` will also have nonconstant coefficients. In order to enforce the relationship
+            between ``sr`` and the current Polynomial, we may require constraints between ``c`` and ``sr.c``.
+            Any such constraints are in this list.
+
+        """
+        # It is important that self._sig_rep.alpha == self.alpha.
+        if self._sig_rep is None:
+            self._compute_sig_rep()
+        sr = self._sig_rep
+        sr_cons = self._sig_rep_constrs
+        return sr, sr_cons
+
     @property
     def grad(self):
         """
@@ -152,7 +207,7 @@ class Polynomial(Signomial):
         ``grad[i](x)`` is the partial derivative of this Polynomial with respect to coordinate ``i``,
         evaluated at ``x``. This array is constructed only when necessary, and is cached upon construction.
         """
-        Signomial._cache_grad(self)
+        Signomial._cache_grad(self)  # This ends up calling the Polynomial "_partial" function.
         return self._grad
 
     @property
@@ -162,17 +217,15 @@ class Polynomial(Signomial):
         ``hess[i,j](x)`` is the (i,j)-th partial derivative of this Polynomial, evaluated at ``x``.
         This array is constructed only when necessary, and is cached upon construction.
         """
-        Signomial._cache_hess(self)
+        Signomial._cache_hess(self)  # This ends up calling the Polynomial "_partial" function.
         return self._hess
 
     def __mul__(self, other):
         if not isinstance(other, Polynomial):
             if isinstance(other, Signomial):  # pragma: no cover
                 raise RuntimeError('Cannot multiply signomials and polynomials.')
-            # else, we assume that "other" is a scalar type
-            tup = (0,) * self.n
-            d = {tup: other}
-            other = Polynomial(d)
+            other = self.upcast_to_signomial(other)
+            other = other.as_polynomial()
         self_var_coeffs = (self.c.dtype not in __NUMERIC_TYPES__)
         other_var_coeffs = (other.c.dtype not in __NUMERIC_TYPES__)
         if self_var_coeffs and other_var_coeffs:  # pragma: no cover
@@ -223,7 +276,7 @@ class Polynomial(Signomial):
     def __pow__(self, power, modulo=None):
         if self.c.dtype not in __NUMERIC_TYPES__:
             raise RuntimeError('Cannot exponentiate polynomials with symbolic coefficients.')
-        temp = Signomial(self.alpha_c)
+        temp = Signomial(self.alpha, self.c)
         temp = temp ** power
         temp = temp.as_polynomial()
         return temp
@@ -245,14 +298,14 @@ class Polynomial(Signomial):
         --------
         Evaluating a Polynomial at a numeric point. ::
 
-            p = Polynomial({(1,2,3): -1})
+            p = Polynomial.from_dict({(1,2,3): -1})
             x = np.array([3,2,1])
             print(p(x))  #  (3 ** 1) * (2 ** 2) * (1 ** 3) * (-1) == -12
 
         Evaluating a Polynomial on another polynomial. ::
 
-            p = Polynomial({(2,): 1})  # represents lambda x: x ** 2
-            z = Polynomial({(1,): 2, (0,): -1})  # represents lambda x: 2*x - 1
+            p = Polynomial.from_dict({(2,): 1})  # represents lambda x: x ** 2
+            z = Polynomial.from_dict({(1,): 2, (0,): -1})  # represents lambda x: 2*x - 1
             w = p(z)  # represents lambda x: (2*x - 1) ** 2
             print(w(0.5))  # equals zero
             print(w(1))  # equals one
@@ -299,16 +352,18 @@ class Polynomial(Signomial):
             res = Signomial.__eq__(self, other)
             return res
 
-    def remove_terms_with_zero_as_coefficient(self):
+    def without_zeros(self):
         """
-        Update ``alpha``, ``c``, and ``alpha_c`` to remove nonconstant terms where ``c[i] == 0``.
+        Return a Polynomial which is symbolically equivalent to ``self``,
+        but which doesn't track basis functions ``alpha[i,:]`` for which ``c[i] == 0``.
         """
-        Signomial.remove_terms_with_zero_as_coefficient(self)
-        pass
+        p = Signomial.without_zeros(self)
+        p = p.as_polynomial()
+        return p
 
     def query_coeff(self, a):
         """
-        Returns the coefficient of the monomial ``lambda x: np.prod(np.power(a, x))`` for this Polynomial.
+        Returns the coefficient of the monomial ``lambda x: np.prod(np.power(x, a))`` for this Polynomial.
         """
         tup = tuple(a)
         if tup in self.alpha_c:
@@ -330,7 +385,7 @@ class Polynomial(Signomial):
         evens = np.where(~(self.alpha % 2).any(axis=1))[0]
         return evens
 
-    def partial(self, i):
+    def _partial(self, i):
         """
         Compute the symbolic partial derivative of this polynomial, at coordinate ``i``.
 
@@ -346,15 +401,20 @@ class Polynomial(Signomial):
         """
         if i < 0 or i >= self.n:
             raise RuntimeError('This polynomial does not have an input at index ' + str(i) + '.')
-        d = defaultdict(int)
+        d = dict()
         for j in range(self.m):
-            if self.alpha[j, i] > 0:
-                vec = self.alpha[j, :].copy()
+            vec = self.alpha[j, :].copy()
+            if vec[i] > 0:
                 c = self.c[j] * vec[i]
                 vec[i] -= 1
-                d[tuple(vec.tolist())] = c
-        d[self.n * (0,)] += 0
-        p = Polynomial(d)
+                tup = tuple(vec)
+                if tup not in d:
+                    d[tup] = c
+                else:
+                    d[tup] += c
+        if len(d) == 0:
+            d[(0,) * self.n] = 0
+        p = Polynomial.from_dict(d)
         return p
 
     def standard_multiplier(self):
@@ -369,31 +429,6 @@ class Polynomial(Signomial):
         mult = Polynomial(mult_alpha, mult_c)
         return mult
 
-    @property
-    def sig_rep(self):
-        """
-        Return the signomial representative of the current Polynomial, as well as a list of constraints needed
-        to enforce the relationship between the current Polynomial and the signomial representative.
-
-        Returns
-        -------
-        sr : Signomial
-            If this Signomial is globally nonnegative, then the current Polynomial is also globally nonnegative.
-
-        sr_cons : list of coniclifts Constraints
-            If the current Polynomial has nonconstant coefficients (i.e. some entries of ``c`` are coniclifts
-            Variables), then ``sr`` will also have nonconstant coefficients. In order to enforce the relationship
-            between ``sr`` and the current Polynomial, we may require constraints between ``c`` and ``sr.c``.
-            Any such constraints are in this list.
-
-        """
-        # It is important that self._sig_rep.alpha == self.alpha.
-        if self._sig_rep is None:
-            self._compute_sig_rep()
-        sr = self._sig_rep
-        sr_cons = self._sig_rep_constrs
-        return sr, sr_cons
-
     def _compute_sig_rep(self):
         self._sig_rep = None
         self._sig_rep_constrs = []
@@ -418,6 +453,8 @@ class Polynomial(Signomial):
             sigrep_c[need_vars] = c_hat
             self._sig_rep_constrs.append(c_hat <= self.c[need_vars])
             self._sig_rep_constrs.append(c_hat <= -self.c[need_vars])
+        if sigrep_c.dtype == object:
+            sigrep_c = cl.Expression(sigrep_c)
         self._sig_rep = Signomial(self.alpha, sigrep_c)
         pass
 
@@ -428,9 +465,34 @@ class Polynomial(Signomial):
         f : Signomial
             For every elementwise positive vector ``x``, we have ``self(x) == f(np.log(x))``.
         """
-        f = Signomial(self.alpha_c)
+        f = Signomial(self.alpha, self.c)
         return f
 
+    def upcast_to_polynomial(self, other):
+        s = Signomial.upcast_to_signomial(self, other)
+        p = s.as_polynomial()
+        return p
+
+    @staticmethod
+    def from_dict(d):
+        """
+        Construct a Polynomial object which represents the function::
+
+            lambda x: np.sum([ d[a] * np.prod(np.power(x, a)) for a in d]).
+
+        Parameters
+        ----------
+        d : Dict[Tuple[Float], Float]
+
+        Returns
+        -------
+        s : Polynomial
+        """
+        s = Signomial.from_dict(d)
+        p = s.as_polynomial()
+        p._alpha_c = s.alpha_c
+        return p
+
 
 class PolyDomain(object):
     """
