@@ -14,8 +14,8 @@
    limitations under the License.
 """
 import sageopt.coniclifts as cl
+from sageopt.symbolic import utilities as sym_util
 import numpy as np
-from collections import defaultdict
 import warnings
 
 
@@ -56,89 +56,20 @@ def standard_sig_monomials(n):
 
 class Signomial(object):
     """
-    A symbolic representation for linear combinations of exponentials, composed with
-    linear functions. The constructor for this class can be called in two different ways, and
-    the arguments to the constructor have names which reflect the different possibilities.
-    Refer to the Examples if you find the description of the constructor arguments unclear.
-
-    Parameters
-    ----------
-
-    alpha_maybe_c : dict or ndarray
-
-         If ``alpha_maybe_c`` is a dict, then it must be a dictionary from tuples-of-numbers to
-         scalars. The keys will be converted to rows of a matrix which we call ``alpha``, and
-         the values will be assembled into a vector which we call ``c``.
-
-         If ``alpha_maybe_c`` is an ndarray, then the argument ``c`` must also be an ndarray,
-         and ``c.size`` must equal the number of rows in ``alpha_maybe_c``.
-
-    c : None or ndarray
-
-        This value is only used when ``alpha_maybe_c`` is an ndarray. If that is the case, then
-        this Signomial will represent the function ``lambda x: c @ np.exp(alpha_maybe_c @ x)``.
-
-    Examples
-    --------
-
-    There are two ways to call the Signomial constructor.
-
-    The first way is to specify a dictionary from tuples to scalars. The tuples are interpreted as linear
-    functionals appearing in the exponential terms, and the scalars are the corresponding coefficients.::
-
-        alpha_and_c = {(1,): 2}
-        f = Signomial(alpha_and_c)
-        print(f(1))  # equal to 2 * np.exp(1).
-        print(f(0))  # equal to 2.
-
-    The second way is to specify two arguments. In this case the first argument is an ndarray
-    where the rows represent linear functionals, and the second argument is a vector of corresponding
-    coefficients.::
-
-        alpha = np.array([[1, 0], [0, 1], [1, 1]])
-        c = np.array([1, 2, 3])
-        f = Signomial(alpha, c)
-        x = np.random.randn(2)
-        print(f(x) - c @ np.exp(alpha @ x))  # zero, up to rounding errors.
-
-    Attributes
-    ----------
-
-    n : int
-        The dimension of the space over which this Signomial is defined. The number of columns in ``alpha``,
-        and the length of tuples appearing in the dictionary ``alpha_c``.
-
-    m : int
-        The number of terms needed to express this Signomial in a basis of monomial functions
-        ``lambda x: exp(a @ x)`` for row vectors ``a``. The signomial is presumed to include a constant term.
-
-    alpha : ndarray
-        Has shape ``(m, n)``. Each row specifies a vector appearing in an exponential function which
-        defines this Signomial. The rows are ordered for consistency with the property ``c``.
-
-    c : ndarray
-        Has shape ``(m,)``. The scalar ``c[i]`` is this Signomial's coefficient on the basis function
-        ``lambda x: exp(alpha[i, :] @ x)``. It is possible to have ``c.dtype == object``.
-
-    alpha_c : defaultdict
-        The keys of ``alpha_c`` are tuples of length ``n``, containing real numeric types (e.g int, float).
-        These tuples define linear functions. This Signomial could be evaluated by the code snippet
-        ``lambda x: np.sum([ alpha_c[a] * np.exp(a @ x) for a in alpha_c])``. The default value for this
-        dictionary is zero.
-
-    Notes
-    -----
+    A concrete representation for a function of the form
+    :math:`x \\mapsto \\sum_{i=1}^m c_i \\exp(\\alpha_i \cdot x)`.
 
     Operator overloading.
 
-        The Signomial class implements ``+``, ``-``, ``*``, and ``**`` between pairs of Signomials,
-        and pairs involving one Signomial and one numeric type.
+        The operators ``+``, ``-``, and ``*`` are defined between pairs of Signomials, and pairs
+        ``{s, t}`` where ``s`` is a Signomial and ``t`` is "scalar-like." Specific examples of
+        "scalar-like" objects include numeric types, and coniclifts Expressions of size one.
 
-        The Signomial class also implements ``s1 / s2`` where ``s2`` is a numeric type or Signomial,
-        but if ``s2`` is a Signomial, then its coefficient vector ``s2.c`` can only contain one nonzero entry.
+        A Signomial ``s`` can be raised to a numeric power ``p`` by writing ``s**p``; if ``s.c``
+        contains more than one nonzero entry, it can only be raised to nonnegative integer powers.
 
-        You can also use ``+``, ``-``, and ``*`` for pairs involving one Signomial and one non-Signomial.
-        If ``s3`` is the result of such an operation, then ``s3.c`` will be an ndarray with ``s3.dtype == object``.
+        The Signomial class implements ``s1 / s2`` where ``s2`` is a numeric type or Signomial;
+        if ``s2`` is a Signomial, then its coefficient vector ``s2.c`` can only contain one nonzero entry.
 
     Function evaluations.
 
@@ -146,65 +77,145 @@ class Signomial(object):
         then ``s(x)`` computes the Signomial object as though it were any other elementary Python function.
 
         Signomial objects provide functions to compute gradients (equivalently, Jacobians) and Hessians.
-        These methods operate by caching and evaluating symbolic representations of the relevant partial derivatives.
+        These methods operate by caching and evaluating symbolic representations of partial derivatives.
 
-    Internal representations.
+    Parameters
+    ----------
 
-        Both ``self.alpha_c`` and ``(self.alpha, self.c)`` completely specify a Signomial object.
+    alpha : ndarray
 
-        ``alpha_c`` is the dictionary which is taken to *define* this Signomial as a mathematical object.
+         The rows of ``alpha`` comprise this Signomial's exponent vectors.
 
-        However, it is useful to have rapid access to the matrix of linear functionals ``alpha``, or the coefficient
-        vector ``c`` as numpy arrays. So these fields are also maintained explicitly.
+    c : None or ndarray
 
-        If a user modifies the fields ``alpha`` or ``c`` directly, there may be an inconsistency between these
-        fields, and the dictionary ``alpha_c``. Therefore the fields ``alpha`` and ``c`` should not be modified
-        without taking great care to ensure consistency with the signomial's dictionary representation.
+        An ndarray of coefficients, with one coefficient for each row in alpha.
+
+    Examples
+    --------
+
+    There are three ways to make Signomial objects. The first way is to call the constructor::
+
+        alpha = np.array([[1, 0], [0, 1], [1, 1]])
+        c = np.array([1, 2, 3])
+        f = Signomial(alpha, c)
+        x = np.random.randn(2)
+        print(f(x) - c @ np.exp(alpha @ x))  # zero, up to rounding errors.
+
+    You can also use ``Signomial.from_dict`` which maps exponent vectors (represented as
+    tuples) to scalars::
+
+        alpha_and_c = {(1,): 2}
+        f = Signomial.from_dict(alpha_and_c)
+        print(f(1))  # equal to 2 * np.exp(1),
+        print(f(0))  # equal to 2.
+
+    The final way to construct a Signomial is with algebraic syntax, like::
+
+        y = sageopt.standard_sig_monomials(2)  # y[i] represents exp(x[i])
+        f = (y[0] - y[1]) ** 3 + 1 / y[0]  # a Signomial in two variables
+        x = np.array([1, 1])
+        print(f(x))  # np.exp(-1), up rounding errors.
+
+    Signomial objects are not limited to numeric problem data for ``alpha`` and ``c``.
+    In fact, it's very common to have ``c`` contain a coniclifts Expression. For example,
+    if we started with a Signomial ``f`` and then updated ::
+
+        gamma = sageopt.coniclifts.Variable()
+        f =  f - gamma
+
+    then ``f.c`` would be a coniclifts Expression depending on the variable ``gamma``.
+
+    Notes
+    -----
+
+    Signomial objects have a dictionary attribute called ``metadata``. You can store any information
+    you'd like in this dictionary. However, the information in this dictionary will not automatically be
+    propogated when creating new Signomial objects (as happens when performing arithmetic on Signomials).
     """
 
-    def __init__(self, alpha_maybe_c, c=None):
-        # noinspection PyArgumentList
-        if c is None:
-            # noinspection PyArgumentList
-            self.alpha_c = defaultdict(int, alpha_maybe_c)
-        else:
-            alpha = np.round(alpha_maybe_c, decimals=__EXPONENT_VECTOR_DECIMAL_POINTS__)
-            alpha = alpha.tolist()
-            if len(alpha) != c.size:  # pragma: no cover
-                raise RuntimeError('alpha and c specify different numbers of terms')
-            self.alpha_c = defaultdict(int)
-            for j in range(c.size):
-                self.alpha_c[tuple(alpha[j])] += c[j]
-        self.n = len(list(self.alpha_c.items())[0][0])
-        self.alpha_c[(0,) * self.n] += 0  # ensures that there's a constant term.
-        self.m = len(self.alpha_c)
-        self.alpha = None
-        self.c = None
+    def __init__(self, alpha, c):
+        if isinstance(alpha, list):
+            alpha = np.ndarray(alpha)
+        if alpha.shape[0] != c.size:  # pragma: no cover
+            raise ValueError('alpha and c specify different numbers of terms')
+        if isinstance(c, np.ndarray) and not isinstance(c, cl.Expression) and c.dtype == object:
+            raise ValueError('If c is an ordinary numpy array, it cannot have dtype == object.')
+        if isinstance(alpha, np.ndarray):
+            alpha = np.round(alpha, decimals=__EXPONENT_VECTOR_DECIMAL_POINTS__)
+            alpha, c = sym_util.consolidate_basis_funcs(alpha, c)
+        self._alpha = alpha
+        self._c = c
+        self._n = alpha.shape[1]
+        self._m = alpha.shape[0]
+        self._alpha_c = None
         self._grad = None
         self._hess = None
-        self._update_alpha_c_arrays()
+        self.metadata = dict()
         pass
 
     def _cache_grad(self):
         if self._grad is None:
-            g = np.empty(shape=(self.n,), dtype=object)
-            for i in range(self.n):
-                g[i] = self.partial(i)
+            g = np.empty(shape=(self._n,), dtype=object)
+            for i in range(self._n):
+                g[i] = self._partial(i)
             self._grad = g
         pass
 
     def _cache_hess(self):
         if self._hess is None:
-            H = np.empty(shape=(self.n, self.n), dtype=object)
-            for i in range(self.n):
-                ith_partial = self.partial(i)
+            H = np.empty(shape=(self._n, self._n), dtype=object)
+            for i in range(self._n):
+                ith_partial = self._partial(i)
                 for j in range(i+1):
-                    curr_partial = ith_partial.partial(j)
+                    curr_partial = ith_partial._partial(j)
                     H[i, j] = curr_partial
                     H[j, i] = curr_partial
             self._hess = H
         pass
 
+    @property
+    def n(self):
+        """
+        The dimension of the space over which this Signomial is defined;
+        this Signomial accepts inputs in :math:`\\mathbb{R}^{n}`.
+        """
+        return self._n
+
+    @property
+    def m(self):
+        """
+        The number of monomial basis functions :math:`x \\mapsto \\exp(a \\cdot x)`
+        used by this Signomial.
+        """
+        return self._m
+
+    @property
+    def alpha(self):
+        """
+        Has shape ``(m, n)``. Each row specifies a vector appearing in an exponential function which
+        defines this Signomial. The rows are ordered for consistency with the property ``c``.
+        """
+        return self._alpha
+
+    @property
+    def c(self):
+        """
+        Has shape ``(m,)``. The scalar ``c[i]`` is this Signomial's coefficient on the basis function
+        ``lambda x: exp(alpha[i, :] @ x)``. It's possible to have ``c.dtype == object``.
+        """
+        return self._c
+
+    @property
+    def alpha_c(self):
+        """
+        The keys of ``alpha_c`` are tuples of length ``n``, containing real numeric types (e.g int, float).
+        These tuples define linear functions. This Signomial could be evaluated by the code snippet
+        ``lambda x: np.sum([ alpha_c[a] * np.exp(a @ x) for a in alpha_c])``.
+        """
+        if self._alpha_c is None:
+            self._build_alpha_c_dict()
+        return self._alpha_c
+
     @property
     def grad(self):
         """
@@ -239,52 +250,45 @@ class Signomial(object):
         """
         Return the index ``i`` so that ``alpha[i, :]`` is the zero vector.
         """
-        return np.where((self.alpha == 0).all(axis=1))[0][0]
+        if not isinstance(self.alpha, np.ndarray):
+            raise NotImplementedError()
+        res = np.where((self.alpha == 0).all(axis=1))[0]
+        if res.size == 0:
+            return None
+        else:
+            return res[0]
 
-    def _update_alpha_c_arrays(self):
-        """
-        Call this function whenever the dictionary representation of this Signomial object has been updated.
-        """
-        alpha = []
-        c = []
-        for k, v in self.alpha_c.items():
-            alpha.append(k)
-            c.append(v)
-        self.alpha = np.array(alpha)
-        self.c = np.array(c)
-        if self.c.dtype == np.dtype('O'):
-            self.c = cl.Expression(self.c)
+    def _build_alpha_c_dict(self):
+        alpha, c = self._alpha, self._c
+        if not isinstance(alpha, np.ndarray):
+            raise NotImplementedError()
+        self._alpha_c = dict()
+        for j in range(c.size):
+            self._alpha_c[tuple(alpha[j, :])] = c[j]
 
     def __add__(self, other):
-        if not isinstance(other, Signomial):
-            tup = (0,) * self.n
-            d = {tup: other}
-            other = Signomial(d)
-        # noinspection PyArgumentList
-        d = defaultdict(int, self.alpha_c)
-        for k, v in other.alpha_c.items():
-            d[k] += v
-        res = Signomial(d)
-        res.remove_terms_with_zero_as_coefficient()
+        try:
+            other = self.upcast_to_signomial(other)
+        except ValueError:
+            return other.__radd__(self)
+        if not other.n == self._n:
+            raise RuntimeError('Cannot add Signomials with different numbers of variables.')
+        res = Signomial.sum([self, other])
+        res = res.without_zeros()
         return res
 
     def __radd__(self, other):
         return self.__add__(other)
 
     def __mul__(self, other):
-        if not isinstance(other, Signomial):
-            tup = (0,) * self.n
-            d = {tup: other}
-            other = Signomial(d)
-        d = defaultdict(int)
-        alpha1, c1 = self.alpha, self.c
-        alpha2, c2 = other.alpha, other.c
-        for i1, v1 in enumerate(alpha1):
-            for i2, v2 in enumerate(alpha2):
-                v3 = np.round(v1 + v2, decimals=__EXPONENT_VECTOR_DECIMAL_POINTS__)
-                d[tuple(v3)] += c1[i1] * c2[i2]
-        res = Signomial(d)
-        res.remove_terms_with_zero_as_coefficient()
+        try:
+            other = self.upcast_to_signomial(other)
+        except ValueError:
+            return other.__rmul__(self)
+        if not other.n == self._n:
+            raise RuntimeError('Cannot multiply Signomials with different numbers of variables.')
+        res = Signomial.product(self, other)
+        res = res.without_zeros()
         return res
 
     def __rmul__(self, other):
@@ -307,29 +311,38 @@ class Signomial(object):
         return other + (-1) * self
 
     def __pow__(self, power, modulo=None):
+        if type(power) not in __NUMERIC_TYPES__:
+            raise RuntimeError('Cannot raise a signomial to non-numeric powers.')
         if self.c.dtype not in __NUMERIC_TYPES__:
-            if isinstance(self.c, cl.Expression) and not self.c.is_constant():  # pragma: no cover
-                raise RuntimeError('Cannot exponentiate signomials with symbolic coefficients.')
+            msg = 'Cannot exponentiate signomials with symbolic coefficients.'
+            if not isinstance(self.c, cl.Expression):
+                raise RuntimeError(msg)
+            elif not self.c.is_constant():
+                raise RuntimeError(msg)
         if power % 1 == 0 and power >= 0:
             power = int(power)
             if power == 0:
                 # noinspection PyTypeChecker
-                return Signomial({(0,) * self.n: 1})
+                return self.upcast_to_signomial(1)
             else:
-                s = Signomial(self.alpha_c)
+                #TODO: implement a faster version of this in utilties.py,
+                # by using multinomial coefficients. Once this is done,
+                # add new tests for correctness of ``__pow__``, since it would
+                # no longer follow from correctness of ``__mul__``.
+                s = Signomial(self.alpha, self.c)
                 for t in range(power - 1):
                     s = s * self
                 return s
         else:
             d = dict((k, v) for (k, v) in self.alpha_c.items() if v != 0)
             if len(d) != 1:
-                raise ValueError('Only signomials with exactly one term can be raised to power %s.')
+                raise ValueError('Only signomials with exactly one term can be raised to power %s.', power)
             v = list(d.values())[0]
             if v < 0 and not power % 1 == 0:
                 raise ValueError('Cannot compute non-integer power %s of coefficient %s', power, v)
             alpha_tup = tuple(power * ai for ai in list(d.keys())[0])
             c = float(v) ** power
-            s = Signomial(alpha_maybe_c={alpha_tup: c})
+            s = Signomial.from_dict({alpha_tup: c})
             return s
 
     def __neg__(self):
@@ -340,19 +353,22 @@ class Signomial(object):
         """
         Evaluates the mathematical function specified by the current Signomial object.
 
-        :param x: either a scalar (if self.n == 1), or a numpy n-d array with len(x.shape) <= 2
-        and x.shape[0] == self.n.
-        :return:  If x is a scalar or an n-d array of shape (self.n,), then "val" is a numeric
-        type equal to the signomial evaluated at x. If instead x is of shape (self.n, k) for
-        some positive integer k, then "val" is a numpy n-d array of shape (k,), with val[i]
+        :param x: either a scalar (if self._n == 1), or a numpy _n-d array with len(x.shape) <= 2
+        and x.shape[0] == self._n.
+        :return:  If x is a scalar or an _n-d array of shape (self._n,), then "val" is a numeric
+        type equal to the signomial evaluated at x. If instead x is of shape (self._n, k) for
+        some positive integer k, then "val" is a numpy _n-d array of shape (k,), with val[i]
         equal to the current signomial evaluated on the i^th column of x.
 
         This function's behavior is undefined when x is not a scalar and has len(x.shape) > 2.
         """
+        if not isinstance(self.alpha, np.ndarray):
+            # assuming cvxpy Expression, could substitute alpha = alpha.value
+            raise NotImplementedError()
         if np.isscalar(x) or (isinstance(x, np.ndarray) and x.size == 1 and x.ndim == 0):
             x = np.array([np.asscalar(np.array(x))])  # coerce into a 1d array of shape (1,).
-        if not x.shape[0] == self.n:
-            msg = 'Domain is R^' + str(self.n) + 'but x is in R^' + str(x.shape[0])
+        if not x.shape[0] == self._n:
+            msg = 'Domain is R^' + str(self._n) + 'but x is in R^' + str(x.shape[0])
             raise ValueError(msg)
         if x.ndim > 2:
             msg = 'Signomials cannot be called on ndarrays with more than 2 dimensions.'
@@ -369,32 +385,43 @@ class Signomial(object):
     def __eq__(self, other):
         if not isinstance(other, Signomial):
             return False
-        if self.m != other.m:
+        if self._m != other._m:
+            return False
+        if self.c.dtype not in __NUMERIC_TYPES__ or other.c.dtype not in __NUMERIC_TYPES__:
+            return False
+        if not isinstance(self._alpha, np.ndarray) or not isinstance(other.alpha, np.ndarray):
             return False
         for k in self.alpha_c:
             v = self.alpha_c[k]
             other_v = other.query_coeff(np.array(k))
-            if not cl.Expression.are_equivalent(other_v, v, rtol=1e-8):
+            if abs(v - other_v) > 1e-8:
                 return False
         return True
 
-    def remove_terms_with_zero_as_coefficient(self):
+    def without_zeros(self):
         """
-        Update ``alpha``, ``c``, and ``alpha_c`` to remove nonconstant terms where ``c[i] == 0``.
+        Return a Signomial which is symbolically equivalent to ``self``,
+        but which doesn't track basis functions ``alpha[i,:]`` for which ``c[i] == 0``.
         """
-        d = dict()
-        for (k, v) in self.alpha_c.items():
-            if (not isinstance(v, __NUMERIC_TYPES__)) or v != 0:
-                d[k] = v
-        # noinspection PyArgumentList
-        self.alpha_c = defaultdict(int, d)
-        tup = (0,) * self.n
-        self.alpha_c[tup] += 0
-        self.m = len(self.alpha_c)
-        self._update_alpha_c_arrays()
-        pass
+        if self.m == 1:
+            return self
+        if isinstance(self.c, cl.Variable):
+            return self
+        to_drop = sym_util.find_zero_entries(self.c)
+        if len(to_drop) == 0:
+            return self
+        elif len(to_drop) == self.m:
+            # noinspection PyTypeChecker
+            return self.upcast_to_signomial(0)
+        else:
+            keepers = np.ones(self.m, dtype=bool)
+            keepers[to_drop] = False
+            c = self.c[keepers]
+            alpha = self.alpha[keepers, :]
+            s = Signomial(alpha, c)
+            return s
 
-    def partial(self, i):
+    def _partial(self, i):
         """
         Compute the symbolic partial derivative of this signomial, at coordinate ``i``.
 
@@ -408,16 +435,17 @@ class Signomial(object):
         p : Signomial
             The function obtained by differentiating this signomial with respect to its i-th argument.
         """
-        if i < 0 or i >= self.n:  # pragma: no cover
+        if i < 0 or i >= self._n:  # pragma: no cover
             raise RuntimeError('This Signomial does not have an input at index ' + str(i) + '.')
-        d = defaultdict(int)
-        for j in range(self.m):
-            c = self.c[j] * self.alpha[j, i]
-            if c != 0:
-                vec = self.alpha[j, :].copy()
-                d[tuple(vec.tolist())] = c
-        d[self.n * (0,)] += 0
-        p = Signomial(d)
+        d = dict()
+        for j in range(self._m):
+            vec = self.alpha[j, :]
+            c = self.c[j] * vec[i]
+            if (not isinstance(c, __NUMERIC_TYPES__)) or c != 0:
+                d[tuple(vec)] = c
+        if len(d) == 0:
+            d[(0,) * self._n] = 0
+        p = Signomial.from_dict(d)
         return p
 
     def grad_val(self, x):
@@ -425,8 +453,8 @@ class Signomial(object):
         Return the gradient of this Signomial (as an ndarray) at the point ``x``.
         """
         _ = self.grad  # construct the function handles.
-        g = np.zeros(self.n)
-        for i in range(self.n):
+        g = np.zeros(self._n)
+        for i in range(self._n):
             g[i] = self._grad[i](x)
         return g
 
@@ -436,8 +464,8 @@ class Signomial(object):
         """
         if self._hess is None:
             _ = self.hess  # ignore the return value
-        H = np.zeros(shape=(self.n, self.n))
-        for i in range(self.n):
+        H = np.zeros(shape=(self._n, self._n))
+        for i in range(self._n):
             for j in range(i+1):
                 val = self._hess[i, j](x)
                 H[i, j] = val
@@ -457,6 +485,94 @@ class Signomial(object):
         f = Polynomial(self.alpha, self.c)
         return f
 
+    def upcast_to_signomial(self, other):
+        if isinstance(other, Signomial):
+            return other
+        alpha = np.zeros(shape=(1, self.n))
+        if isinstance(other, __NUMERIC_TYPES__):
+            s = Signomial(alpha, np.array([other]))
+            return s
+        elif isinstance(other, cl.base.ScalarExpression):
+            s = Signomial(alpha, cl.Expression([other]))
+            return s
+        elif not hasattr(other, 'size'):
+            raise ValueError()
+        elif other.size > 1:
+            raise ValueError()
+        else:
+            s = Signomial(alpha, other.flatten())
+            return s
+
+    @staticmethod
+    def product(f1, f2):
+        alpha1, c1 = f1.alpha, f1.c
+        if isinstance(alpha1, np.ndarray):
+            alpha1 = alpha1.astype(np.float_)
+        m1 = alpha1.shape[0]
+        alpha2, c2 = f2.alpha, f2.c
+        if isinstance(alpha2, np.ndarray):
+            alpha2 = alpha2.astype(np.float_)
+        m2 = alpha2.shape[0]
+        # lift alpha1, c1 into the product basis
+        tile_idxs = np.tile(np.arange(m1), reps=m2)
+        alpha1_lift = alpha1[tile_idxs, :]
+        c1_lift = c1[tile_idxs]
+        # lift alpha2, c2 into the product basis
+        repeat_idxs = np.repeat(np.arange(m2), repeats=m1)
+        alpha2_lift = alpha2[repeat_idxs, :]
+        c2_lift = c2[repeat_idxs]
+        # explicitly form the product basis, and the coefficient vector for the product
+        alpha_lift = alpha1_lift + alpha2_lift
+        if isinstance(alpha_lift, np.ndarray):
+            alpha_lift = np.round(alpha_lift,
+                                  decimals=__EXPONENT_VECTOR_DECIMAL_POINTS__)
+        c_lift = c1_lift * c2_lift
+        p = type(f1)(alpha_lift, c_lift)
+        return p
+
+    @staticmethod
+    def sum(funcs):
+        if len(funcs) == 0:
+            raise ValueError()
+        elif any(not isinstance(f, Signomial) for f in funcs):
+            raise ValueError()
+        elif len(funcs) == 1:
+            return funcs[0]
+        mats = [f.alpha for f in funcs]
+        alpha, all_crs = sym_util.align_basis_matrices(mats)
+        num_rows = alpha.shape[0]
+        coeff_vecs = [f.c for f in funcs]
+        lifted_cs = sym_util.lift_basis_coeffs(coeff_vecs, all_crs, num_rows)
+        c = sum(lifted_cs)
+        s = type(funcs[0])(alpha, c)
+        return s
+
+    @staticmethod
+    def from_dict(d):
+        """
+        Construct a Signomial object which represents the function::
+
+            lambda x: np.sum([ d[a] * np.exp(a @ x) for a in d])
+
+        Parameters
+        ----------
+        d : Dict[Tuple[Float], Float]
+
+        Returns
+        -------
+        s : Signomial
+        """
+        alpha = []
+        c = []
+        for k, v in d.items():
+            alpha.append(k)
+            c.append(v)
+        alpha = np.array(alpha)
+        c = np.array(c)
+        s = Signomial(alpha, c)
+        s._alpha_c = d
+        return s
+
 
 class SigDomain(object):
     """
@@ -535,6 +651,7 @@ class SigDomain(object):
         self.check_feas = kwargs['check_feas'] if 'check_feas' in kwargs else True
         self._constraints = None  # optional
         self._x = None  # optional
+        self._lift_x = None  # for computing support function
         self._variables = None  # optional
         if 'AbK' in kwargs:
             self.A, self.b, self.K = kwargs['AbK']
@@ -550,8 +667,9 @@ class SigDomain(object):
 
     def _check_feasibility(self):
         A, b, K = self.A, self.b, self.K
-        temp_x = cl.Variable(shape=(A.shape[1],), name='temp_x')
-        cons = [cl.PrimalProductCone(A @ temp_x + b, K)]
+        if self._lift_x is None:
+            self._lift_x = cl.Variable(shape=(A.shape[1],), name='temp_x')
+        cons = [cl.PrimalProductCone(A @ self._lift_x + b, K)]
         prob = cl.Problem(cl.MIN, cl.Expression([0]), cons)
         prob.solve(verbose=False, solver='ECOS')
         if not prob.value < 1e-7:
@@ -628,3 +746,25 @@ class SigDomain(object):
         if any([abs(g(x_val)) > tol for g in self.eqs]):
             return False
         return True
+
+    def suppfunc(self, y):
+        """
+        The support function of the convex set :math:`X` associated with this SigDomain,
+        evaluated at :math:`y`:
+
+        .. math::
+
+            \\sigma_X(y) \\doteq \\max\\{ y^\\intercal x \\,:\\, x \\in X \\}.
+        """
+        if isinstance(y, cl.Expression):
+            y = y.value
+        if self._lift_x is None:
+            self._lift_x = cl.Variable(self.A.shape[1])
+        objective = y @ self._lift_x
+        cons = [cl.PrimalProductCone(self.A @ self._lift_x + self.b, self.K)]
+        prob = cl.Problem(cl.MAX, objective, cons)
+        prob.solve(solver='ECOS', verbose=False)
+        if prob.status == cl.FAILED:
+            return np.inf
+        else:
+            return prob.value
